# -*- coding: utf-8 -*-
{% if print_urls %}from django.conf.urls import url{% elif print_paths %}from django.conf.urls import path{% endif %}
from django.views.generic import TemplateView
from .views import {{ model_name }}ListView, {{ model_name }}CreateView, {{ model_name }}UpdateView, {{ model_name }}DeleteView, {{ model_name }}DetailView

urlpatterns = [{% if print_urls %}
    url(r'^$', {{ model_name }}ListView.as_view(), name='list'),
    url(r'^create/$', {{ model_name }}CreateView.as_view(), name='create'),
    url(r'^update/(?P<pk>\d+)/$', {{ model_name }}UpdateView.as_view(), name='update'),
    url(r'^delete/(?P<pk>\d+)/$', {{ model_name }}DeleteView.as_view(), name='delete'),
    url(r'^detail/(?P<pk>\d+)/$', {{ model_name }}DetailView.as_view(), name='detail'),{% elif print_paths %}
    path('', {{ model_name }}ListView.as_view(), name='list'),
    path('create/', {{ model_name }}CreateView.as_view(), name='create'),
    path('update/<int:pk>', {{ model_name }}UpdateView.as_view(), name='update'),
    path('delete/<int:pk>', {{ model_name }}DeleteView.as_view(), name='delete'),
    path('detail/<int:pk>', {{ model_name }}DetailView.as_view(), name='detail'),{% endif %}
]
